#!/usr/bin/python3
"""
Tray icon tool with a TOTP implementation.
Usage:
totptray [title=key]
"""
import sys
import pyperclip
import pystray
import pyotp
from PIL import Image, ImageDraw

def _create_icon():
    """
    Creates a T-shaped icon.

    :returns: A 64x64 image with "T" in a circle.
    """
    icon_size = 64
    t_width = int(0.3 * icon_size)
    t_height = int(0.4 * icon_size)
    horizontal_margin = int((icon_size - t_width) / 2)
    vertical_margin = int((icon_size - t_height) / 2)

    points = (
        (horizontal_margin, vertical_margin),
        (icon_size - horizontal_margin, vertical_margin),
        (icon_size / 2, vertical_margin),
        (icon_size / 2, icon_size - vertical_margin))

    line_size = int(0.1 * icon_size)
    image = Image.new('RGBA', (icon_size, icon_size), (0, 0, 0, 0))

    draw = ImageDraw.Draw(image)
    draw.ellipse([(0, 0), (icon_size, icon_size)], fill="white")
    draw.line(points, fill="black", width=line_size)

    return image

def _copy_code(key):
    """
    Copies a TOTP generated code for the given key to the clipboard.
    :param key: For for which a TOTP code should be generated.
    """
    code = pyotp.TOTP(key).now()
    pyperclip.copy(code)

def _create_menu(keys):
    """
    Creates a list of menu items for each passed key.
    :param keys: A list of lists with 2 items each. The first item is a label, the second a key.
    :returns: A list of menu items for each passed key.
    """
    assert all(len(key) == 2 for key in keys)
    return [pystray.MenuItem(key[0], lambda _: _copy_code(key[1])) for key in keys]

def main():
    """
    Creates a tray icon with a menu that allows to copy OTP generated codes.
    """
    assert len(sys.argv) > 1
    assert all(item.count('=') == 1 for item in sys.argv[1:])
    icon = pystray.Icon('totptray', icon=_create_icon(), menu=_create_menu([x.split('=') for x in sys.argv[1:]]))
    icon.run()

if __name__ == '__main__':
    main()



import pathlib
from setuptools import setup

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# This call to setup() does all the work
setup(
    name="totptray",
    version="1.1.0",
    description="A simple tray icon tool with a TOTP implementation.",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/McCzarny/totptray",
    author="Maciej Czarnecki",
    author_email="mcczarny@gmail.com",
    license="The Unlicense",
    classifiers=[
        "License :: OSI Approved :: The Unlicense (Unlicense)",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
    ],
    packages=["totptray"],
    include_package_data=True,
    install_requires=["pyperclip", "pystray", "pyotp"],
    entry_points={
        "console_scripts": [
            "totptray=totptray.__main__:main",
        ]
    },
)



#!/usr/bin/python3
from totptray import totptray
import unittest
import pyperclip

class Test(unittest.TestCase):
    def test_icon_creation(self):
        icon = totptray._create_icon()
        try:
            icon.verify()
        except:
            self.fail("Icon does not pass verification!")

        self.assertTrue(icon.width > 1)
        self.assertTrue(icon.height > 1)
    
    def test_copy_code(self):
        #Clear the clipboard.
        pyperclip.copy("")
        try:
            totptray._copy_code("test")
        except:
            self.fail("Got an exception while calling _copy_code!")

        code = pyperclip.paste()
        self.assertEqual(len(code), 6)
        self.assertTrue(code.isdigit())

    def test_create_menu_empty(self):
        menu = totptray._create_menu("")
        self.assertEqual(len(menu), 0)

    def test_create_menu_one(self):
        menu = totptray._create_menu((("label","key"),))
        self.assertEqual(len(menu), 1)
        self.assertEqual(menu[0].text, "label")

    def test_create_menu_multiple(self):
        menu = totptray._create_menu((("label0","key0"),("label1","key1"),("label2","key2")))
        self.assertEqual(len(menu), 3)
        for idx, item in enumerate(menu):
            self.assertEqual(item.text, "label" + str(idx))

    def test_create_menu_incorrect(self):
        self.assertRaises(AssertionError, totptray._create_menu, (("label=key",),))

if __name__ == '__main__':
    unittest.main()
